package obu;

import java.util.logging.Filter;
import java.util.logging.Logger;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import org.yaml.snakeyaml.Yaml;

public class ShutTheHellUpPlugin extends JavaPlugin
{
    public static final Logger log = Logger.getLogger("Minecraft");
    public static PluginDescriptionFile pdfFile = null;
    public static ArrayList<Map> filterList = null;
    public static Filter oldFilter = null;
    public static ObuFilter obuFilter = new ObuFilter();
    public static Timer timer = new Timer("Obu log filter checker");
    public static boolean activateTimer = false;
    public static int timerPeriod = 0;
    public static int timerDelay = 5;

    public void onEnable()
    {
        pdfFile = this.getDescription();

        loadFilters();
        Filter currentFilter = log.getFilter();
        if (currentFilter != null && currentFilter.getClass().getName() != "obu.ObuFilter")
        {
            oldFilter = currentFilter;
        }
        if (currentFilter != obuFilter)
            log.setFilter(obuFilter);

        if (activateTimer == true)
        {
            if (timerPeriod > 0) this.timer.schedule(new suTimerTask(),(long) timerDelay*1000,(long) timerPeriod*1000);
            else this.timer.schedule(new suTimerTask(),(long) timerDelay*1000);
        }


        log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Enabled!" );
    }

    public void loadFilters()
    {
        InputStream yml = null;
        try
        {
            File folder = new File("plugins/ObuShutTheHellUp");
            if (!folder.exists())
            {
                folder.mkdirs();
            }

            File file = new File("plugins/ObuShutTheHellUp/filters.yml");
            if (!file.exists())
            {
                // create file from scratch
                log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Filter file not found! Creating default filter file.");
                String fileContent =
                        "{\n" +
                        "# ObuShutTheHellUp has a timed functionality that deals with incompatibilities with other plugins, this fixes other plugins that 'steal' the log filter.\n" +
                        "  activateTimer: false,\n" +
                        "# timerDelay > 0:  fix runs for the first time after <timerDelay> seconds\n" +
                        "  timerDelay: 5,\n" +
                         "# timerPeriod = 0: turned off - runs only once\n" +
                        "# timerPeriod > 0: turned on - fix runs every <timerPeriod> seconds after the first time\n" +
                        "  timerPeriod: 0,\n" +
                        "# type:\n" +
                        "#    'string' - log entry is filtered if this value is contained in the message's text.\n" +
                        "#    'regex' - log entry is filtered if the message's text matches at least once in the regular expression.\n" +
                        "# value:\n" +
                        "#    the value to be compared (either by string comparison or by regular expression)\n" +
                        "# level:\n" +
                        "#    'INFO'/'WARNING'/'FATAL'/'ANY'/etc - the value of the log level) \n" + 
                        "    filters: [\n" +
                        "        { type: 'string', value: 'overloaded?', level: 'WARNING' }\n" +
                        "    ]\n" +
                        "}";

                Writer writer = new FileWriter("plugins/ObuShutTheHellUp/filters.yml");
                writer.write(fileContent);
                writer.close();
            }

            Yaml yaml = new Yaml();
            Reader reader = new FileReader("plugins/ObuShutTheHellUp/filters.yml");
            Map<String, Object> data = (Map<String, Object>) yaml.load(reader);

            filterList = (ArrayList<Map>) data.get("filters");
            if (data.containsKey("activateTimer"))
            {
                activateTimer = Boolean.parseBoolean(data.get("activateTimer").toString());
            }
            else activateTimer = false;
            if (data.containsKey("timerDelay"))
            {
                timerDelay = Integer.parseInt(data.get("timerDelay").toString());
            }
            else timerDelay = 5;
            if (data.containsKey("timerPeriod"))
            {
                timerPeriod = Integer.parseInt(data.get("timerPeriod").toString());
            }
            else timerPeriod = 0;
            
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Settings file successfully loaded");
        }
        catch(Exception ex)
        {
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Exception while trying to read filters file - File format is bad");
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Message: " + ex.getMessage());
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] " + ex.toString());
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Controlled stack trace bellow:");
            ex.printStackTrace();
            log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Loading default behaviour instead (filtering server can't keep up log only)");

            HashMap map = new HashMap();
            map.put("type", "string");
            map.put("value", "overloaded?");
            map.put("level", "WARNING");
            filterList = new ArrayList<Map>();
            filterList.add(map);
            
        }
    }

    public void onDisable()
    {
        if (activateTimer == true)
        {
            // clears the timer
            timer.cancel();
            timer.purge();
        }

        if (log.getFilter() != null && log.getFilter().getClass().getName() == "obu.ObuFilter")
        {
            // removes the current filter
            log.setFilter(null);
        }

        log.info("[" + pdfFile.getName() + " version " + pdfFile.getVersion() + "] Disabled!" );
    }
    
}
